/**
 * OW2 Util
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: EnhancedCluePoolFactory.java 5224 2009-11-08 14:00:33Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.pool.impl.enhanced;

import java.util.concurrent.Executor;

import org.ow2.util.pool.impl.enhanced.api.IPool;
import org.ow2.util.pool.impl.enhanced.api.thread.IReusableThread;
import org.ow2.util.pool.impl.enhanced.impl.thread.ThreadPool;
import org.ow2.util.pool.impl.enhanced.impl.util.ExecutorProvider;
import org.ow2.util.pool.impl.enhanced.internal.resizer.impl.shared.SharedResizerPoolThreadManager;
import org.ow2.util.pool.impl.enhanced.manager.clue.ICluePoolManager;


/**
 *
 * @author Gael Lalire
 */
public class EnhancedCluePoolFactory {

    /**
     * Pool of reusable thread.
     */
    private IPool<IReusableThread> pool;

    public EnhancedCluePoolFactory(final IPool<IReusableThread> pool) {
        this.pool = pool;
    }

    /**
     * You should use this method to create {@link EnhancedCluePool}.
     * @param <E> pool item type
     * @param <C> clue type
     * @param cluePoolManager a clue pool manager
     * @return a new clue pool
     */
    public <E, C> EnhancedCluePool<E, C> createEnhancedCluePool(final ICluePoolManager<E, C> cluePoolManager) {
        return createEnhancedCluePool(cluePoolManager, false, true);
    }

    /**
     *
     * @param <E> pool item type
     * @param <C> clue type
     * @param cluePoolManager a clue pool manager
     * @param setSizeShared if true setExpectiveSize will be called in a separate thread, you
     * should prefers the schedulerShared option because on a remove call the creation
     * will be on self thread.
     * @param schedulerShared each time a pool item is needed a shared executor will be
     * used
     * @return a new clue pool
     */
    public <E, C> EnhancedCluePool<E, C> createEnhancedCluePool(final ICluePoolManager<E, C> cluePoolManager,
            final boolean setSizeShared, final boolean schedulerShared) {
        Executor executor;
        if (schedulerShared) {
            executor = new ThreadPool(pool, Long.valueOf(0L));
        } else {
            executor = ExecutorProvider.SELF_THREAD_EXECUTOR;
        }
        if (setSizeShared) {
            return new EnhancedCluePool<E, C>(cluePoolManager, ResizerType.SHARED_ASYNCHRONOUS,
                    new SharedResizerPoolThreadManager(pool), executor);
        } else {
            return new EnhancedCluePool<E, C>(cluePoolManager, ResizerType.SIMPLE, null, executor);
        }
    }


}
